/**@file

// plcConfigIcemonPlc.ctl
This library contains the function to configure the IcemonPlc.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// plcConfigIcemonPlc_checkConfig
/** check the device configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
plcConfigIcemonPlc_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  if (dynlen(dsConfigs) >= UN_CONFIG_COMMON_LENGTH)
  {
  // 1. Common
    unConfigGenericFunctions_checkParameters(dsConfigs, "IcemonPlc", exceptionInfo);
  // 2. call the device check function
    unConfigGenericFunctions_checkDeviceConfig(dsConfigs, "IcemonPlc", exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","plcConfigConfigIcemonPlc_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// plcConfigIcemonPlc_setConfig
/** set the device configuration
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigs  input, config line as dyn_string
@param exceptionInfo output, for errors
*/
plcConfigIcemonPlc_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName;
  
// 1. Initialization : checking + dpCreate
  unConfigGenericFunctions_checkAll(dsConfigs, "IcemonPlc", exceptionInfo);
  if (dynlen(exceptionInfo) <= 0)    // Good inputs
  {
    sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
    unConfigGenericFunctions_createDp(sDpName, "IcemonPlc", exceptionInfo);
    if (dynlen(exceptionInfo) <= 0)  // Creation ok or dp already exists
    {
// 2. Generate device
// 2.1. Parameters
      unConfigGenericFunctions_setCommon(dsConfigs, exceptionInfo);
  // 2.2 call the device check function
      unConfigGenericFunctions_setDeviceConfig(dsConfigs, "IcemonPlc", exceptionInfo);
    }    
  }
}

//------------------------------------------------------------------------------------------------------------------------
// CMW_IcemonPlc_checkConfig
/** check the IcemonPlc device configuration for the CMW front-end
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
CMW_IcemonPlc_checkConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  
  bool bOk=false;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc import CMW_IcemonPlc", "CMW_IcemonPlc_checkConfig", dsConfigs);
  if (dynlen(dsConfigs) == (PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_COMMON_LENGTH))
    bOk = true;
  else if (dynlen(dsConfigs) == (PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH)) {
    bOk = true;
    unConfigGenericFunctions_checkArchive(dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL],
                                          dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG],
                                          dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_COMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT],
                                          exceptionInfo);
  }
  else
    bOk = false;
  
  if(bOk) {
    // check all the device configuration
    // if the archive config is defined check if the archive config is Ok 
    //unConfigGenericFunctions_checkArchiveParameters(0, dsConfigs[UN_CONFIG_COMMON_LENGTH + PLC_CONFIG_ICEMONPLC_ARCHIVEACTIVE], exceptionInfo);
  }
  else
  {
    fwException_raise(exceptionInfo,"ERROR","CMW_IcemonPlc_checkConfig: " + getCatStr("unGeneration","BADINPUTS"),"");
  }
}

//------------------------------------------------------------------------------------------------------------------------
// CMW_IcemonPlc_setConfig
/** set the IcemonPlc device configuration for the CMW front-end
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@param dsConfigs input, config line as dyn_string
@param exceptionInfo output, for errors
*/
CMW_IcemonPlc_setConfig(dyn_string dsConfigs, dyn_string &exceptionInfo)
{
  string sDpName, sDpe, sConfig, sPLCName;
  bool archiveActive;
  int i, iDriverNumber, iEnd;
  string sPLCType, sPrefix;
  string sGroup, sFilterName, sFilterValue, sCMWDevice, sCMWProperty1, sCMWProperty2, sCMWProperty3, sCMWTag;
  bool bLLC;
  
  sDpName = dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME];
  iDriverNumber = dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER];
  
   
  sPLCType = dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PLC_CONFIG_ICEMONPLC_TYPE];
  sPLCName = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS];
 
  bLLC=TRUE;
  sCMWProperty1="PLCALARMS";
  sCMWProperty2="PLCINFO"; 
  sCMWProperty3="PLCDIAG";   
  
  sCMWTag="";
  sFilterValue="";
  sFilterName="";
  sGroup="";
  sCMWDevice="MONITOR_"+sPLCName; 
  plcConfig_setCommon(sDpName, sPLCType, dsConfigs, sPrefix, iEnd, exceptionInfo);
  
  //set subscription 
  for(i=1;(i<=iEnd) && (dynlen(exceptionInfo)==0);i++)
  { 
    sDpe=sDpName + ".ProcessInput.metric["+i+"]";
    sCMWTag=getCatStr("plcMonitoring", sPrefix + i); 
    unConfigCMW_setAddress(sDpe, iDriverNumber, DPATTR_ADDR_MODE_INPUT_SPONT,
                          CMW_LONG_TYPE, sGroup, sFilterName, sFilterValue, 
                          sCMWDevice, sCMWProperty1, sCMWTag, bLLC, exceptionInfo);
  } 
  unConfigCMW_setAddress(sDpName + ".ProcessInput.PLCtime", iDriverNumber, DPATTR_ADDR_MODE_INPUT_SPONT,
                          CMW_STRING_TYPE, sGroup, sFilterName, sFilterValue, 
                          sCMWDevice, sCMWProperty1, sPrefix + "TIME", bLLC, exceptionInfo); 
  unConfigCMW_setAddress(sDpName + ".ProcessInput.PLCinfo", iDriverNumber, DPATTR_ADDR_MODE_INPUT_SPONT,
                          CMW_STRING_TYPE, sGroup, sFilterName, sFilterValue, 
                          sCMWDevice, sCMWProperty2, sPrefix + "INFO", bLLC, exceptionInfo); 
  unConfigCMW_setAddress(sDpName + ".ProcessInput.PLCdiag", iDriverNumber, DPATTR_ADDR_MODE_INPUT_SPONT,
                          CMW_STRING_TYPE, sGroup, sFilterName, sFilterValue, 
                          sCMWDevice, sCMWProperty3, sPrefix + "DIAG", bLLC, exceptionInfo); 
  
  //set pseudo.dev
  unConfigCMW_setAddress(sDpName + ".ProcessInput.valid", iDriverNumber, DPATTR_ADDR_MODE_INPUT_SPONT,
                          CMW_BOOL_TYPE, sGroup, sFilterName, sFilterValue, 
                          sCMWDevice, sCMWProperty1 + ":pseudo.dev", "contents", bLLC, exceptionInfo);

}

//------------------------------------------------------------------------------------------------------------------------
// IcemonPlc_getArchiveDPE
/** return the list of IcemonPlc DPE per archive (Bool, Ana, Event)
  
@par Constraints
  None

@par Usage
  Internal

@par PVSS managers
  Ui, CTRL

@return output, list of DPE
*/
/* Comment this function if you want to use the default setting: 
   Bool archive: bool, char DPE
   Ana archive: float DPE, int and uint DPE except evStsReg*
   Event archive: evStsReg* int and uint, struct DPE
*/
dyn_string IcemonPlc_getArchiveDPE()
{
  dyn_string dsRet;
  dyn_string dsArchiveElements, exceptionInfo, dsDpel;
  dyn_int diDpelType;
  int i, len, j, k, length, lengthk, pos;
  string archiveBoolNames, archiveEventNames, archiveAnalogNames;
  string sDeviceType="IcemonPlc";
  
//  dsRet[1] = ".ProcessInput.DPE1"+ ";"+".ProcessInput.DPE2";
//  dsRet[2] = archiveAnalogNames;
//  dsRet[3] = archiveEventNames;
  
  // the following code can also be used
  // in this example:
  // Bool archive: bool, char DPE
  // Ana archive: float DPE, blob DPE, all int DPE except evStsReg* 
  // Event archive: evStsReg* int, uint, struct DPE
     
  // get all the DPE having an archive config from the JCOP device definition
  fwDevice_getArchiveElements(sDeviceType, dsArchiveElements, exceptionInfo);
  // get all the DPE from the device type
  unGenericDpFunctions_getDPE(sDeviceType, dsDpel, diDpelType);
  
  length = dynlen(dsArchiveElements);
  lengthk = dynlen(dsDpel);
//DebugN(length, lengthk);
  for(j=1;j<=length;j++)
  {
    for(k=1;k<=lengthk;k++) {
//DebugN(dsDpel[k], dsArchiveElements[j]);
      if((sDeviceType+dsArchiveElements[j]) == dsDpel[k]) {
        switch(diDpelType[k]){
          case DPEL_CHAR:
          case DPEL_BOOL:
            archiveBoolNames = archiveBoolNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_FLOAT:
            archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_INT:
            // if DPE like evStsReg --> put it in event archive
            if (patternMatch(".ProcessInput.evStsReg*", dsArchiveElements[j]))
            {
              archiveEventNames = archiveEventNames + dsArchiveElements[j] + ";";
            }
            else
            {
              archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            }
            break;
          case DPEL_UINT:
          case DPEL_BOOL_STRUCT:
            archiveEventNames = archiveEventNames + dsArchiveElements[j] + ";";
            break;
          case DPEL_BLOB:
            archiveAnalogNames = archiveAnalogNames + dsArchiveElements[j] + ";";
            break;
        }
      }
    }
  }
  dsRet[1] = archiveBoolNames;
  dsRet[2] = archiveAnalogNames;
  dsRet[3] = archiveEventNames;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc import IcemonPlc", "IcemonPlc_getArchiveDPE", dsRet);
  return dsRet;
}

//------------------------------------------------------------------------------------------------------------------------
// IcemonPlc_getArchiveProxyDPE
/** return a dyn_dyn_string with the list of proxy DPE linked (or that will be linked) 
to the device for the three archives: boolean, analog and event

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param dsConfigLine input, the device configuration
@return the list of DPE per archive (3)
*/
dyn_dyn_string IcemonPlc_getArchiveProxyDPE(dyn_string dsConfigLine)
{
  dyn_dyn_string dds;
  
  //return here for the boolean archive the list of proxy DPEs linked to the device
  dds[UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] = makeDynString();
  //return here for the analog archive the list of proxy DPEs linked to the device
  dds[UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] = makeDynString();
  //return here for the event archive the list of proxy DPEs linked to the device
  dds[UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] = makeDynString();
  
  return dds;
}

void plcConfig_setCommon(const string sDpName, const string sPLCType, const dyn_string dsConfigs, string &sPrefix, int &iEnd, dyn_string &exceptionInfo)
{
  string sDpe, description;
  dyn_bool dbOkLogic;
  int i;
  dyn_string dsAlertText, dsAlertClass;
  dyn_string dsDpElement, dsAlertPanelParameters, dsDescription, dsSplit;
  dyn_float dfLimits;
  string sAlertPanel, sHelp, sDomainConfig, sDomain, sNatureConfig, sNature, sDeviceAlias, sDeviceLinks;
  // if archive name are in the config->overwrite panel setting. 
  if(dynlen(dsConfigs) == (PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH))
  {
    if(dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL] = dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL];
    if(dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG];
    if(dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] != "")
      dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = dsConfigs[PLC_CONFIG_ICEMONPLC_LENGTH + UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT];
  }
  //Set Description for Alert List 

  sDomainConfig = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DOMAIN];
  dsSplit = strsplit(sDomainConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  // access control + domain present 
  if(dynlen(dsSplit) == UN_CONFIG_UNICOS_DOMAIN_AND_ACCESS_CONTROL_LEN) sDomain = dsSplit[2];
  else if(strpos(sDomainConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR) >= 0) sDomain = ""; // if UN_ACCESS_CONTROL_DOMAIN_SEPARATOR in sDomain config --> domain = ""
  else sDomain = sDomainConfig;
  
  sNatureConfig = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_NATURE];
  dsSplit = strsplit(sNatureConfig, UN_ACCESS_CONTROL_DOMAIN_SEPARATOR);
  // if no ACC priv assume list of nature
  if(dynlen(dsSplit) >= (UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN -1))
  { 
    if(dynlen(dsSplit) == UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN) sNature = dsSplit[4];
    else sNature = ""; // dynlen = UN_CONFIG_UNICOS_NATURE_AND_PRIV_LEN -1 --> consider nature = ""
  }
  else sNature = sNatureConfig;
  
  dsDescription [UN_CONFIG_ALARM_DOMAIN] = sDomain;
  dsDescription [UN_CONFIG_ALARM_NATURE] = sNature;
  unConfigGenericFunctions_getAliasLink(dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS], sDeviceAlias, sDeviceLinks);
  dsDescription [UN_CONFIG_ALARM_ALIAS] = sDeviceAlias;
  dsDescription [UN_CONFIG_ALARM_DESCRIPTION] = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION];
  dsDescription [UN_CONFIG_ALARM_NAME] = "PLC Agent"; 
  dsDescription [UN_CONFIG_ALARM_DEFAULTPANEL] = dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DEFAULTPANEL]; 
  unConfigGenericFunctions_setAlarmDescription(sDpName + ".ProcessInput.alert", dsDescription, exceptionInfo); 
  
  //set Alerts
  dsAlertText=makeDynString(UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_OK, UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_BAD,"Invalid");
  dsAlertClass=makeDynString("", PLC_CONFIG_DEFAULT_ALERT_CLASS, "_unNotValid");
  dfLimits=makeDynFloat(-1,0,1);
  dsAlertPanelParameters=makeDynString();
  sAlertPanel="";
  sHelp="";
  fwAlertConfig_delete(sDpName + ".ProcessInput.metric", exceptionInfo); //delete OLD Alert Summary 
  fwAlertConfig_delete(sDpName + ".ProcessInput.alert", exceptionInfo); //delete Alert Summary
  dynClear(dsDpElement);
  switch (sPLCType)
  {
    case SCHNEIDER_PLC_PREMIUM:
    case SCHNEIDER_PLC_QUANTUM: 
      dbOkLogic=makeDynBool(FALSE, FALSE, FALSE, TRUE, FALSE, TRUE, TRUE, TRUE);
      sPrefix="SC_";
      iEnd=PLC_SCHNEIDER_METRIC_LENGTH;
      break;
    case SIEMENS_PLC_S7_400:
    case SIEMENS_PLC_S7_300: 
      dbOkLogic=makeDynBool(FALSE, FALSE, TRUE, TRUE, TRUE);
      sPrefix="SI_";
      iEnd=PLC_SIEMENS_METRIC_LENGTH; 
      break;
    default:
      fwException_raise(exceptionInfo, "ERROR", "PLC Type unknown: " + sPLCType + "("+SCHNEIDER_PLC_PREMIUM+","+SCHNEIDER_PLC_QUANTUM+","+SIEMENS_PLC_S7_400+","+"SIEMENS_PLC_S7_300"+")", "");
  }
  for(i=1;i<=iEnd;i++)
  { 
    if(dynlen(exceptionInfo)==0)
    {
      if(dbOkLogic[i])
      {
        dfLimits = makeDynFloat(1,0,-1);
      }
      else
      {
        dfLimits = makeDynFloat(0, 1,-1);
      }
      sDpe=sDpName + ".ProcessInput.metric["+i+"]";
      fwAlertConfig_setDiscrete(sDpe, DPCONFIG_ALERT_NONBINARYSIGNAL, dsAlertText, dfLimits, dsAlertClass, sAlertPanel, dsAlertPanelParameters, sHelp, exceptionInfo, FALSE, false, false, true);
      fwAlertConfig_activate(sDpe, exceptionInfo, TRUE, TRUE);
      dynAppend(dsDpElement, sDpe); 
      description =  getCatStr("plcMonitoring", getCatStr("plcMonitoring", sPrefix + i)) + " in " + sDeviceAlias;
      unConfigGenericFunctions_setDescription(sDpe, description, exceptionInfo);
      unConfigGenericFunctions_setAlias(sDpe, description, exceptionInfo);
    }
  } 
  if(dynlen(exceptionInfo)==0)
  {
    dsAlertText=makeDynString(UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_OK, UN_CONFIG_DEFAULT_ALERT_DIGITAL_TEXT_BAD); 
    fwAlertConfig_setSummary(sDpName + ".ProcessInput.alert", dsAlertText, dsDpElement, sAlertPanel, dsAlertPanelParameters, sHelp, exceptionInfo, FALSE);
    if(dynlen(exceptionInfo)==0)
      fwAlertConfig_activate(sDpName + ".ProcessInput.alert", exceptionInfo, TRUE, FALSE); 
  }
  //manage error
  if(dynlen(exceptionInfo)>0)
  {
    for(i=1;i<=PLC_ICEMONPLC_METRIC_LENGTH;i++)
      fwAlertConfig_delete(sDpName + ".ProcessInput.metric[" + i + "]", exceptionInfo);
  } 
  else
  { 
    //set Archives
    dynClear(dsDpElement);
    for(i=1;i<=iEnd;i++)
    {
      dynAppend(dsDpElement,sDpName + ".ProcessInput.metric[" + i + "]"); 
    }
    //dynAppend(dsDpElement,sDpName + ".ProcessInput.PLCtime"); 
    dynAppend(dsDpElement,sDpName + ".ProcessInput.PLCinfo");
    dynAppend(dsDpElement,sDpName + ".ProcessInput.PLCdiag"); 
    dynAppend(dsDpElement,sDpName + ".ProcessInput.HWConfig");
    dynAppend(dsDpElement,sDpName + ".statusInformation.deviceLog"); 
    fwArchive_setMultiple(dsDpElement, dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], DPATTR_ARCH_PROC_SIMPLESM, DPATTR_COMPARE_OLD_NEW, 0, 0, exceptionInfo, TRUE);

    //Set config 
    dpSetWait(sDpName + ".Configuration.type", sPLCType,
              sDpName + ".Configuration.IP", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PLC_CONFIG_ICEMONPLC_IP],
              sDpName + ".Configuration.application", dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + PLC_CONFIG_ICEMONPLC_APPLICATION]);              
  }
}

//------------------------------------------------------------------------------------------------------------------------

//@}
