/**@file

// plcIcemonPlc.ctl
This library contains the widget, faceplate, etc. functions of IcemonPlc.

@par Creation Date
  dd/mm/yyyy

@par Modification History
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@author 
  the author (DEP-GROUP)
*/

//@{

//------------------------------------------------------------------------------------------------------------------------
// plcIcemonPlc_AcknowledgeAlarm
/** Returns the list of IcemonPlc DPE with alarm config that can be acknowledged
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName  input, device name DP name
@param sDpType input, device type
@param dsNeedAck output, the lsit of DPE
*/
plcIcemonPlc_AcknowledgeAlarm(string sDpName, string sDpType, dyn_string &dsNeedAck)
{
  dsNeedAck = makeDynString("ProcessInput....");
  // if the acknoledge must be sent to the PLC: CPC Front-end: add UN_ACKNOWLEDGE_PLC
  //dsNeedAck = makeDynString("ProcessInput....", UN_ACKNOWLEDGE_PLC);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc AcknowledgeAlarm IcemonPlc "+sDpName, "plcIcemonPlc_AcknowledgeAlarm", sDpName, sDpType, dsNeedAck);
}

//------------------------------------------------------------------------------------------------------------------------
// plcIcemonPlc_ObjectListGetValueTime
/** Function called from snapshot utility of the treeDeviceOverview to get the time and value
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDeviceName  input, device name
@param sDeviceType input, device type
@param dsReturnData output, return data, array of 5 strings
*/
plcIcemonPlc_ObjectListGetValueTime(string sDeviceName, string sDeviceType, dyn_string &dsReturnData)
{
  string sTime, sValue, sState, sColor;
  bool bOk, bBad;
  
  dpGet(sDeviceName+".ProcessInput....", sValue, 
        sDeviceName+".ProcessInput....", sState, 
        sDeviceName+".ProcessInput....:_online.._stime", sTime, 
        sDeviceName+".ProcessInput....:_online.._bad", bBad);

  dsReturnData[1] = sTime;
  dsReturnData[2] = sValue;
  dsReturnData[3] = bBad;
  dsReturnData[4] = sState;
  dsReturnData[5] = sColor;
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc snapshot IcemonPlc "+sDeviceName, "plcIcemonPlc_ObjectListGetValueTime", sDeviceName, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_FaceplateStatusRegisterCB
/** faceplate DistributedControl callback of the faceplate status panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
plcIcemonPlc_FaceplateStatusRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction, iRes;
  dyn_string exceptionInfo;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;
                                         
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc faceplate IcemonPlc "+sDp, "plcIcemonPlc_FaceplateStatusRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      plcIcemonPlc_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_DPCONNECT:
      plcIcemonPlc_FaceplateConnect(deviceName);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      plcIcemonPlc_FaceplateDisconnect(deviceName);
      plcIcemonPlc_FaceplateStatusDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_FaceplateStatusDisconnection
/** set the faceplate when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
plcIcemonPlc_FaceplateStatusDisconnection()
{
// 2. animate FE Connection
  plcGenericFunctions_FaceplateStatusDisconnect();
  // there is no need to do here execScript because there is one faceplate per device type
  // set all the graphical element to the state data not connnected
  // ex. of functions that can be used
  //unGenericObject_ColorBoxDisconnect, unGenericObject_DisplayValueDisconnect, etc.
  unGenericObject_ColoredSquareDisconnect(makeDynString("m1", "m2", "m3", "m4", "m5", "m6", "m7", "m8"));
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc faceplate IcemonPlc", "plcIcemonPlc_FaceplateStatusDisconnection");
}
void plcIcemonPlc_FaceplateInit(const string sType)
{
  string sPrefix;
  int i, iLen;
  switch(sType)
  {
    case SCHNEIDER_PLC_PREMIUM  :
    case SCHNEIDER_PLC_QUANTUM:
      sPrefix = "SC_";
      iLen = PLC_SCHNEIDER_METRIC_LENGTH;
      break;
    case SIEMENS_PLC_S7_300:
    case SIEMENS_PLC_S7_400:
      sPrefix = "SI_";
      iLen = PLC_SIEMENS_METRIC_LENGTH;
      break;
  }
  if (sPrefix != "")
  {
    for (i = 1; i <= iLen; i++)
    {
      plcGenericFunctions_TextBoxAnimate(i, sPrefix+i);    
    }  
    
    for (i = PLC_ICEMONPLC_METRIC_LENGTH; i >= (iLen + 1);i--)
    {
      setMultiValue("m"+ i + ".body", "visible", false, "m"+i + ".border", "visible", false, "t" + i, "visible", false);
    }
  }
}
//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_FaceplateConnect
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
plcIcemonPlc_FaceplateConnect(string deviceName)
{
  int iAction, iRes;
  dyn_string exceptionInfo;
  string sFrontEnd, sSystemName, sType;
  
  g_sDeviceAlias = unGenericDpFunctions_getAlias(deviceName);

  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
  g_sFrontEndDp = sSystemName+sFrontEnd;
  g_sFrontEndAlias = unGenericDpFunctions_getAlias(g_sFrontEndDp);
  
  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }

  dpGet(deviceName + ".Configuration.type", sType);
  plcIcemonPlc_FaceplateInit(sType);  
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc faceplate IcemonPlc "+deviceName, "plcIcemonPlc_FaceplateConnect", deviceName, sFrontEnd, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {    
    // set all faceplate global variable usefull for the callback, e.g.: description, format, unit, etc.
    // set graphical element of the faceplate to the init state if needed
    
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the faceplate and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the plcIcemonPlc_FaceplateDisconnect
    iRes = dpConnect("plcIcemonPlc_FaceplateStatusAnimationCB",
                      deviceName + ".statusInformation.deviceLog",
                      deviceName + ".ProcessInput.metric[1]",
                      deviceName + ".ProcessInput.metric[2]",
                      deviceName + ".ProcessInput.metric[3]",
                      deviceName + ".ProcessInput.metric[4]",                      
                      deviceName + ".ProcessInput.metric[5]",
                      deviceName + ".ProcessInput.metric[6]",
                      deviceName + ".ProcessInput.metric[7]",
                      deviceName + ".ProcessInput.metric[8]",
                      deviceName + ".ProcessInput.PLCinfo",
                      deviceName + ".ProcessInput.PLCdiag",
                      deviceName + ".ProcessInput.HWConfig",
                      deviceName + ".ProcessInput.PLCtime",
                      deviceName + ".ProcessInput.valid",
                      deviceName + ".Configuration.type",
                      deviceName + ".Configuration.IP",
                      deviceName + ".Configuration.application",
                      sFrontEnd+".alarm",
                      sFrontEnd+".enabled");
    g_bCallbackConnected = (iRes >= 0);
  }
  else
    plcIcemonPlc_FaceplateStatusDisconnection();
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_FaceplateDisconnect
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
*/
plcIcemonPlc_FaceplateDisconnect(string deviceName)
{
  int iRes;
  string sFrontEnd, sSystemName;
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc faceplate IcemonPlc "+deviceName, "plcIcemonPlc_FaceplateDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc) {
// get the systemName
    sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
    sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;

    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the faceplate set during plcIcemonPlc_FaceplateConnect
    // and do to the correct dpDisconnect
     iRes = dpConnect("plcIcemonPlc_FaceplateStatusAnimationCB",
                      deviceName + ".statusInformation.deviceLog",
                      deviceName + ".ProcessInput.metric[1]",
                      deviceName + ".ProcessInput.metric[2]",
                      deviceName + ".ProcessInput.metric[3]",
                      deviceName + ".ProcessInput.metric[4]",                      
                      deviceName + ".ProcessInput.metric[5]",
                      deviceName + ".ProcessInput.metric[6]",
                      deviceName + ".ProcessInput.metric[7]",
                      deviceName + ".ProcessInput.metric[8]",
                      deviceName + ".ProcessInput.PLCinfo",
                      deviceName + ".ProcessInput.PLCdiag",
                      deviceName + ".ProcessInput.HWConfig",
                      deviceName + ".ProcessInput.PLCtime",
                      deviceName + ".ProcessInput.valid",
                      deviceName + ".Configuration.type",
                      deviceName + ".Configuration.IP",
                      deviceName + ".Configuration.application",
                      sFrontEnd+".alarm",
                      sFrontEnd+".enabled");
    g_bCallbackConnected = !(iRes >= 0);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_FaceplateStatusAnimationCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
plcIcemonPlc_FaceplateStatusAnimationCB(string sDp, dyn_string dsLog, 
                                                    string sDp1, int b1,
                                                    string sDp2, int b2,
                                                    string sDp3, int b3,
                                                    string sDp4, int b4,
                                                    string sDp5, int b5,
                                                    string sDp6, int b6,
                                                    string sDp7, int b7,
                                                    string sDp8, int b8,
                                                    string sDp9, dyn_string dsInfo,
                                                    string sDp10, dyn_string dsDiag,
                                                    string sDp11, dyn_string dsHW,
                                                    string sDpTime, string sTime,   
                                                    string sDpValid, bool bValid,
                                                    string sDpConfig1, string sType,
                                                    string sDpConfig2, string sIP,
                                                    string sDpConfig3, string sApplication,
                                                    string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                                                    string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
 

  string lastUpdate;
  dyn_string dsValue, dsDpe;
  dyn_int diValue;
  int i, iLen;
  dyn_bool dbOkLogic;
  bool bInvalid = !bValid;
  if(g_bSystemConnected)
    plcGenericFunctions_FaceplateStatusHeader(g_sFrontEndDp, g_sFrontEndAlias, g_sDeviceAlias, iFESystemIntegrityAlarmValue, 
                                              bFESystemIntegrityAlarmEnabled, bInvalid);
  // animate all the graphical element of the faceplate
  // all the variables global to the faceplate can be used
  // encapsulate all the setValue or function call with the following if(g_bSystemConnected)
  /* e.g.:
    if(g_bSystemConnected)
      unGenericObject_DisplayValue(...);
    */
  // ex. of functions that can be used: unGenericObject_DisplayValue, unGenericObject_ColorBoxAnimate, 
  // unGenericObject_ColorBoxAnimateAlarm, unGenericObject_ColoredSquareAnimate, 
  // unGenericObject_DisplayRange, etc.
  
  time t = getCurrentTime();
  lastUpdate = formatTime("%d/%m/%y %H:%M:%S", t);
  setValue("txtrefresh", "text", "PLC current time: " + sTime);
  setValue("txtrefreshtime", "text", "Last update: " + lastUpdate);
  
  dsValue = makeDynString(sIP, sType, sApplication);
  if (dsValue != g_OldsValue)
  {
    setMultiValue("txtIP.display", "text", sIP, "txtType.display", "text", sType, "txtApplication.display", "text", sApplication);
    g_OldsValue = dsValue;
  }  
  
  if (dsInfo != g_OldsInfo)
  {
    iLen = dynlen(dsInfo);
    for(i = iLen; i >= 1; i--)
    {
      if (dsInfo[i] == "") dynRemove(dsInfo, i);    
    }
    txtinfo.deleteAllItems();
    setValue("txtinfo", "items", dsInfo);
    g_OldsInfo = dsInfo;
  }
  
  if (dsDiag != g_OlddsDiag)
  {
    iLen = dynlen(dsDiag);
    for (i = iLen; i >= 1; i--)
    {
      if (dsDiag[i] == "") dynRemove(dsDiag, i);    
    }
    txtdiag.deleteAllItems();
    setValue("txtdiag", "items", dsDiag);
    g_OlddsDiag = dsDiag;
  }
  
  if(dsHW != g_OlddsHW)
  {
    setValue("txthw", "text", dsHW);
    g_OlddsHW = dsHW;  
  }
  
  dsDpe = makeDynString(sDp1, sDp2, sDp3, sDp4, sDp5, sDp6, sDp7, sDp8);
  diValue = makeDynString(b1, b2, b3, b4, b5, b6, b7, b8);
  
  if (diValue != g_OlddiValue || dsLog != g_OlddsLog)
  {
    switch(sType)
    {
      case SCHNEIDER_PLC_PREMIUM:
      case SCHNEIDER_PLC_QUANTUM:
        dbOkLogic = makeDynBool(false, false, false, true, false, true, true, true);
        iLen = PLC_SCHNEIDER_METRIC_LENGTH;
        break;
      case SIEMENS_PLC_S7_400:
      case SIEMENS_PLC_S7_300:
        dbOkLogic = makeDynBool(false, false, true, true, true);
        iLen = PLC_SIEMENS_METRIC_LENGTH;
        break;
      default:
        iLen = 0;
    }  

    
    for(i =1 ; i<= iLen; i++)
    {
      plcGenericFunctions_ColorBoxAnimateAlarm(dsDpe[i], i, diValue[i], dbOkLogic[i], bInvalid);
    }
    g_OlddiValue = diValue;
    g_OlddsLog = dsLog;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_ButtonRegisterCB
/** Contextual button DistributedControl callback of the contextual device button panel
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
plcIcemonPlc_ButtonRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sDpType, sFunction;
  int iAction, iRes, iManReg01;
  dyn_string exceptionInfo, dsFunctions, dsUserAccess;
  
  deviceName = unGenericDpFunctions_getDpName($sDpName);
  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+deviceName, "plcIcemonPlc_ButtonRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch (iAction)
  {
    case UN_ACTION_DISCONNECT:
      plcIcemonPlc_ButtonDisconnection();
      break;
    case UN_ACTION_DPCONNECT:

      sDpType="IcemonPlc";
      unGenericObject_GetFunctions(sDpType, dsFunctions, exceptionInfo);
      sFunction = dsFunctions[UN_GENERICOBJECT_FUNCTION_BUTTONUSERACCESS];
             
      if (sFunction != "")
      {
        evalScript(dsUserAccess, "dyn_string main(string deviceNameTemp, string sDpTypeTemp) {" + 
               "dyn_string dsUserAccessReturn;" +
               "if (isFunctionDefined(\"" + sFunction  + "\"))" +
               "    {" +
                 sFunction  + "(deviceNameTemp, sDpTypeTemp, dsUserAccessReturn);" +
               "    }" +
               "return dsUserAccessReturn; }", makeDynString(), deviceName, sDpType);
        // keep the list of the device allowed action 
        g_dsUserAccess=dsUserAccess;  
      }
      else
        fwException_raise(exceptionInfo,"ERROR", "plcIcemonPlc_ButtonRegisterCB:" + getCatStr("unGeneration","UNKNOWNFUNCTION"), "EMPTY");

    // set all contextual panel global variable usefull for the plcIcemonPlc_ButtonAnimationCB, e.g.: description, format, unit, etc.
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+deviceName, "plcIcemonPlc_ButtonRegisterCB Connect", deviceName, sDpType, dsUserAccess);
      iRes = dpConnect("plcIcemonPlc_ButtonAnimationCB",
               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
               deviceName + ".statusInformation.selectedManager");
      g_bCallbackConnected = (iRes >= 0);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+deviceName, "plcIcemonPlc_ButtonRegisterCB Disconnect", deviceName);
      iRes = dpDisconnect("plcIcemonPlc_ButtonAnimationCB",
               deviceName + ".statusInformation.selectedManager:_lock._original._locked",
               deviceName + ".statusInformation.selectedManager");
      g_bCallbackConnected = !(iRes >= 0);
      plcIcemonPlc_ButtonDisconnection();
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_UserLoginGetButtonState
/** return the device data, this function is called when the user logs in and logs out
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sType input, the device type
@param dsReturnData output, the device data, [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
plcIcemonPlc_UserLoginGetButtonState(string deviceName, string sType, dyn_string &dsReturnData)
{
  int iRes;
  string sDpLock, sDpSelectedManager, sSelectedManager;
  bool bLocked;

  // read the device data (6 string)
  sDpLock = deviceName + ".statusInformation.selectedManager:_lock._original._locked";
  sDpSelectedManager = deviceName + ".statusInformation.selectedManager";
  if (dpExists(deviceName))
  {
    iRes = dpGet(sDpLock, bLocked, sDpSelectedManager, sSelectedManager);
  }
  else
  {
    iRes = -1;
  }
  if (iRes < 0) // Errors during dpget
  {
    bLocked = false;
    sSelectedManager = "";
  }
  dsReturnData=makeDynString(bLocked, sSelectedManager, "..", "..", "..", "..");
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+deviceName, "plcIcemonPlc_UserLoginGetButtonState", deviceName, sType, dsReturnData);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_ButtonSetState
/** Set the state of the contextual button of the device
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, the device DP name
@param sDpType input, the device type
@param dsUserAccess input, list of allowed action on the device
@param dsData input, the device data [1] = lock state, [2] = lock by, [3] .. [6] device data
*/
plcIcemonPlc_ButtonSetState(string deviceName, string sDpType, dyn_string dsUserAccess, dyn_string dsData)
{
  bool bSelected;
  dyn_string dsButtons;
  int i, length;
  string localManager;
  bool bLocked, buttonEnabled;
  string sSelectedManager, sDeviceValue1, sDeviceValue2, sDeviceValue3, sDeviceValue4;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+deviceName, "plcIcemonPlc_ButtonSetState", deviceName, sDpType, dsUserAccess, dsData);
  bLocked = dsData[1];
  sSelectedManager = dsData[2];
  sDeviceValue1 = dsData[3];
  sDeviceValue2 = dsData[4];
  sDeviceValue3 = dsData[5];
  sDeviceValue4 = dsData[6];

  // dsButtons = list of all the possible actions except Select 
  // use the name of the graphical element without "button"
  // use the name of the graphical elements.
  dsButtons = makeDynString("Alerts"); 
  length = dynlen(dsButtons);
// 1. Selection state
  localManager = unSelectDeselectHMI_getSelectedState(bLocked, sSelectedManager);
  bSelected = (localManager == "S");
// 2. for each possible action check if the action is in the list dsUserAccess
  // crosscheck if needed with the state of the device
  for(i=1;i<=length;i++)
  {
    buttonEnabled = (dynContains(dsUserAccess, dsButtons[i]) > 0);  // User access
    switch(dsButtons[i])
    {
      case "Alerts":
        // calculate the enable state of the button from the device data
        buttonEnabled = buttonEnabled & bSelected;
        break;
      default:
        break;
    }
    setValue(dsButtons[i], "enabled", buttonEnabled);
  }
// 3. Button Select state
  unGenericObject_ButtonAnimateSelect(localManager, (dynContains(dsUserAccess, UN_FACEPLATE_BUTTON_SELECT) > 0));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_ButtonDisconnection
/** set the state of the device action button when the device system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

*/
plcIcemonPlc_ButtonDisconnection()
{
  // set all the device action button to disabled
  unGenericObject_ButtonDisconnect(makeDynString(UN_FACEPLATE_BUTTON_SELECT, "Alerts"));
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc", "plcIcemonPlc_ButtonDisconnection");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_ButtonUserAccess
/** returns the list of allowed action on the device for a user logged in
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, the device DP name
@param sDpType input, the device type
@param dsAccess input, list of allowed action on the device
*/
plcIcemonPlc_ButtonUserAccess(string sDpName, string sDpType, dyn_string &dsAccess)
{
  bool operator, expert, admin;
  int i, length;
  dyn_bool dbPermissions;
  dyn_string exceptionInfo;
  string sOperAction, sExpAction, sAdminAction;
  bool bActionDefined = false;
  
  if(isFunctionDefined("fwSysOverview_getACDomain"))
  {
    dyn_string exceptionInfo;
    bool hasExpertPrivilege=FALSE;
    string domain;
    fwSysOverview_getACDomain(domain, sDpName);
  
    if(domain == "") //AC not yet configured. Enable access 
    {
      hasExpertPrivilege = true;
    }
    else
    {
      string expertPrivilege = fwSysOverview_getExpertPrivilege(2);
      // check the current  privilege level 
      if (expertPrivilege == "")
      {
        DebugN("No expert privilege defined!");
        return;
      }
      fwAccessControl_isGranted(domain + ":" +expertPrivilege, hasExpertPrivilege, exceptionInfo);
    
      // check and handle exceptions that might have been thrown by the function above
      if (dynlen(exceptionInfo)) {fwAccessControl_displayException(exceptionInfo);return;};
      if(hasExpertPrivilege) 
      {
        dynAppend(dsAccess, makeDynString("Alerts", UN_FACEPLATE_BUTTON_SELECT));
      }
    }
  }
  else
  {
    // get the configured list of authorized action per privilege
    unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
    if((sOperAction != UNICOS_ACCESSCONTROL_DOMAINNAME) || (sExpAction != UNICOS_ACCESSCONTROL_DOMAINNAME) ||
       (sAdminAction != UNICOS_ACCESSCONTROL_DOMAINNAME))
      bActionDefined=true;
    operator = dbPermissions[2];
    expert = dbPermissions[3];
    admin = dbPermissions[4];
    dsAccess = makeDynString();
  
    if(admin){
      if(bActionDefined)
        dynAppend(dsAccess, strsplit(sAdminAction, UN_ACCESS_CONTROL_SEPARATOR));
      else
        dynAppend(dsAccess, makeDynString("Alerts"));
    }
  }

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+sDpName, "plcIcemonPlc_ButtonUserAccess", sDpName, sDpType, dbPermissions, sOperAction, sExpAction, sAdminAction, dsAccess);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_ButtonAnimationCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLock input, the lock DPE 
@param bLocked input, selected lock state of the device
@param sDpSelect input, the select DPE 
@param sSelectedManager input, user and Ui that selected the device
@param sDp1 input, the DPE1 
@param blobData input, device data
@param sDp2 input, the DPE2 
@param iData input, device status data
@param sDpInvalid input, the invalid DPE 
@param bInvalid input, data validity
*/
plcIcemonPlc_ButtonAnimationCB(string sDpLock, bool bLocked, string sDpSelect, string sSelectedManager)
{
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc button IcemonPlc "+sDpLock, "moniIcemonPlc_ButtonAnimationCB", g_dsUserAccess);
  // call the set button state with the device data and the current allowed action on the device
  // which is in g_dsUserAccess
  plcIcemonPlc_ButtonSetState(unGenericDpFunctions_getDpName(sDpLock), "IcemonPlc", g_dsUserAccess, 
                  makeDynString(bLocked, sSelectedManager, "..", "..", "..", ".."));
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_WidgetRegisterCB
/** widget DistributedControl callback
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDp input, the DistributedControl system name DP name
@param bSystemConnected input, the state of the system name
*/
plcIcemonPlc_WidgetRegisterCB(string sDp, bool bSystemConnected)
{
  string deviceName, sSystemName;
  int iAction;
  dyn_string exceptionInfo;
  string sFrontEnd;
  bool bRemote;
  
  deviceName = unGenericDpFunctions_getWidgetDpName($sIdentifier);
  if (deviceName == "")  // In case of disconnection
  {
    deviceName = g_sDpName;
  }
  g_bUnSystemAlarmPlc = false;
// get the systemName
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
// get the PLC name 
  sFrontEnd = unGenericObject_GetPLCNameFromDpName(deviceName);

  unDistributedControl_isRemote(bRemote, sSystemName);
  if(bRemote)
    g_bSystemConnected = bSystemConnected;
  else
    g_bSystemConnected = true;

  if(sFrontEnd != ""){
    sFrontEnd = sSystemName+c_unSystemAlarm_dpPattern+DS_pattern+sFrontEnd;
//DebugN(sSystemName, sFrontEnd);
    if(dpExists(sFrontEnd)) {
      g_bUnSystemAlarmPlc = true;
    }
  }

  unGenericObject_Connection(deviceName, g_bCallbackConnected, bSystemConnected, iAction, exceptionInfo);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc widget IcemonPlc "+deviceName, "plcIcemonPlc_WidgetRegisterCB", sDp, deviceName, g_bCallbackConnected, iAction);
  switch(iAction)
  {
    case UN_ACTION_DISCONNECT:
      plcIcemonPlc_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_DPCONNECT:
      g_sDpName = deviceName;
      plcIcemonPlc_WidgetConnect(deviceName, sFrontEnd);
      break;
    case UN_ACTION_DPDISCONNECT_DISCONNECT:
      plcIcemonPlc_WidgetDisconnect(deviceName, sFrontEnd);
      plcIcemonPlc_WidgetDisconnection(g_sWidgetType);
      break;
    case UN_ACTION_NOTHING:
    default:
      break;
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_WidgetDisconnection
/** set the widget when the system is disconnected 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
plcIcemonPlc_WidgetDisconnection(string sWidgetType)
{
  /** use the exec mechanism to allow multiple widget. **/
  string sFunction = "plcIcemonPlc_"+sWidgetType+"Disconnection";

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc widget IcemonPlc", "plcIcemonPlc_WidgetDisconnection", sWidgetType);
  if (isFunctionDefined(sFunction) && (sFunction != ""))
  {
    execScript("main(string sWidgetType) {" + sFunction + "(sWidgetType);}", makeDynString(), sWidgetType);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_WidgetConnect
/** dpConnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name
*/
plcIcemonPlc_WidgetConnect(string deviceName, string sFrontEnd)
{
  int iRes;
  string sSystemName, sType;
  
//  check if the device name is of type "IcemonPlc" if not like no front-end. -> not available.
  if(dpTypeName(deviceName) != "IcemonPlc")
    g_bUnSystemAlarmPlc = false;
  
  sSystemName = unGenericDpFunctions_getSystemName(deviceName);
  
  dpGet(deviceName + ".Configuration.type", sType);
  plcIcemonPlc_WidgetInit(sType);  
  
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc widget IcemonPlc "+deviceName, "plcIcemonPlc_WidgetConnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // set all widget global variable usefull for the callback, e.g.: description, format, unit, etc.
    
    // if there are many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // keep the kind of dpConnect in a variable global to the widget and do a swith for the dpConnect
    // and do to the correct dpDisconnect in the plcIcemonPlc_WidgetDisconnect

    string fsmDp; 
    if (isDollarDefined("$1") && isDollarDefined("$2") && $1 != "" && $2 != "")
    {
      fsmDp = $1+"|" + $2;
    }
    else
    {
      string alias = unGenericDpFunctions_getAlias(deviceName);
       //resolve fsm dp
      dyn_string fsmDps = dpNames("*|"+alias, "_FwFsmDevice");
      if (dynlen(fsmDps) > 0)
        fsmDp = fsmDps[1];
    }  
    iRes = dpConnect("plcIcemonPlc_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          deviceName + ".ProcessInput.alert:_alert_hdl.._act_text",
                          deviceName + ".Configuration.type",
                          deviceName + ".ProcessInput.valid",      
                          fsmDp + ".fsm.currentState",                    
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bCallbackConnected = (iRes >= 0);    
  }else{
    plcIcemonPlc_WidgetDisconnection(g_sWidgetType);
  }
}

void plcIcemonPlc_WidgetInit(string sType)
{
  string sPicture, sColor;
  sColor = "unSynopticBackground";
  switch (sType)
  {
    case SIEMENS_PLC_S7_400:
      sPicture = "logo/siemens_s7-400.png";
      break;
    case SIEMENS_PLC_S7_300:
      sPicture = "logo/siemens_s7-300.png";
      break;
    case SCHNEIDER_PLC_QUANTUM:
      sPicture = "logo/schneider_quantum.png";
      break;
    case SCHNEIDER_PLC_PREMIUM:
      sPicture = "logo/schneider_premium.png";
      break;
    default:
      sPicture = "logo/plc.png";    
  }
  
  setMultiValue("body", "backCol", sColor, "picture", "fill", "[pattern,[tile,png," + sPicture + "]]");
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_WidgetDisconnect
/** dpDisconnect to the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, front-end device DP name
@param sFrontEnd input, front-end device name

*/
plcIcemonPlc_WidgetDisconnect(string deviceName, string sFrontEnd)
{
  int iRes;

  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc widget IcemonPlc "+deviceName, "plcIcemonPlc_WidgetDisconnect", deviceName, g_bUnSystemAlarmPlc);
  if(g_bUnSystemAlarmPlc)
  {
    // if there were many dpConnect possibilities: e.g. dpConnect with or without _alert_hdl config
    // re-use the kind of dpConnect kept in a variable global to the widget set during plcIcemonPlc_WidgetConnect
    // and do to the correct dpDisconnect
    iRes = dpDisconnect("plcIcemonPlc_WidgetCB", 
                          deviceName + ".statusInformation.selectedManager:_lock._original._locked",
                          deviceName + ".statusInformation.selectedManager",
                          deviceName + ".ProcessInput.alert:_alert_hdl.._alert_color",
                          deviceName + ".Configuration.type",                          
                          deviceName + ".ProcessInput.valid",
                          sFrontEnd+".alarm",
                          sFrontEnd+".enabled");
    g_bCallbackConnected = !(iRes >= 0);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_WidgetCB
/** callback function on the device data
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpLocked input, the lock DPE 
@param bLocked input, the lock state 
@param sDpSelectedManager input, the selected manager DPE 
@param sSelectedManager input, the selected manager 
@param sDp1 input, the DPE1 
@param blobData input, device data
@param sDpValid input, the valid DPE 
@param bValid input, data validity
@param sDpFESystemIntegrityAlarmValue input, front-end device system integrity alarm DPE 
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param sDpFESystemIntegrityAlarmEnabled input, front-end device system integrity enable DPE
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
plcIcemonPlc_WidgetCB(string sDpLocked, bool bLocked,
                                  string sDpSelectedManager, string sSelectedManager,
                                  string sDp1, string sAlert, 
                                  string sDpConf, string sType,
                                  string sDpValid, bool bValid,
                                  string sDpFsm, string fsmState,
                                  string sDpFESystemIntegrityAlarmValue, int iFESystemIntegrityAlarmValue, 
                                  string sDpFESystemIntegrityAlarmEnabled, bool bFESystemIntegrityAlarmEnabled)
{
  /** use the exec mechanism to allow multiple widget. **/
  string sFunction = "plcIcemonPlc_"+g_sWidgetType+"Animation";

  string selectColor, sWarningLetter, sWarningColor, sDpN;
  bool bSelectVisible;
  bool bInvalid = !bValid;
  sDpN = unGenericDpFunctions_getDpName(sDp1);

  // 1. set the widget.
  unGenericObject_WidgetSelectAnimation(bLocked, sSelectedManager, selectColor, bSelectVisible);
  
// if Front-end problem or problems-> O letter
  // !!! Logic inversed (FE and then Invalid) compared to other UNICOS devices
    if((!bFESystemIntegrityAlarmEnabled) || (iFESystemIntegrityAlarmValue > c_unSystemIntegrity_no_alarm_value))
    {
      sWarningLetter = UN_WIDGET_TEXT_OLD_DATA;
      sWarningColor = "unDataNotValid";
    }
  else
    {
    // if bInvalid --> N letter and everything blue      
    if (bInvalid)
      {
      sWarningLetter = UN_WIDGET_TEXT_INVALID;
      sWarningColor = "unDataNotValid";
      }      
  }
  if(g_bSystemConnected)
    setMultiValue("WarningText", "text", sWarningLetter, 
                  "WarningText", "foreCol", sWarningColor,
                  "SelectArea", "foreCol", selectColor,
                  "LockBmp", "visible", bSelectVisible,
                  "WidgetArea", "visible", true);


  if (isFunctionDefined(sFunction) && (sFunction != ""))
  {
    // set here all the common graphical element value or
    // give to the function all the variables
    execScript("main(string sDpN, string sData, string sType,string sDpFsm, string fsmState, bool bSystemConnected, string sWidgetType, int iFESystemIntegrityAlarmValue, bool bFESystemIntegrityAlarmEnabled) {" + 
               sFunction + "(sDpN, sData, sType, sDpFsm, fsmState, bSystemConnected, sWidgetType, iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);}", 
               makeDynString(), sDpN, sAlert, sType, sDpFsm, fsmState, g_bSystemConnected, g_sWidgetType, iFESystemIntegrityAlarmValue, bFESystemIntegrityAlarmEnabled);
  }
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_MenuConfiguration
/** pop-up menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sDpName input, device DP name
@param sDpType input, device type
@param dsAccessOk input, the access control 
@param menuList output, pop-up menu to show, dyn_string to be given to the popupMenu function
*/
plcIcemonPlc_MenuConfiguration(string sDpName, string sDpType, dyn_string dsAccessOk, dyn_string &menuList)
{
  dyn_string dsMenuConfig = makeDynString(UN_POPUPMENU_SELECT_TEXT, UN_POPUPMENU_FACEPLATE_TEXT, UN_POPUPMENU_TREND_TEXT);
  
  unGenericObject_addSelectToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  unGenericObject_addDefaultUnicosActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList);
  if (isFunctionDefined("icemon_addMoonActionsToMen"))
    icemon_addMoonActionsToMen(menuList);
  //unGenericObject_addTrendActionToMenu(sDpName, sDpType, dsMenuConfig, dsAccessOk, menuList, true);
  // to add a device type menu add the menu from UN_POPUPMENU_MIN
  /** e.g.: device menu enable/disabled depending on the device access right
  int pos, iMenu=UN_POPUPMENU_MIN;
  string sAccess, sOperAction, sExpAction, sAdminAction, sMenu;
  dyn_string exceptionInfo, dsArchiveName;
  dyn_bool dbPermissions;
  
  pos = dynContains(menuList, "PUSH_BUTTON, Device configuration, 10, 1");
  if(pos > 0) {
  // get the access rigth on the device
    unGenericDpFunctions_getAccessControlPriviledgeRigth(sDpName, g_dsDomainAccessControl, dbPermissions, sOperAction, sExpAction, sAdminAction, exceptionInfo);
    sAccess = "0";
    if(dbPermissions[4]) {
      sAccess = "1";
    }

    sprintf(sMenu, "%d", (iMenu+1));
    dynInsertAt(menuList, "SEPARATOR", pos);
    dynInsertAt(menuList, "PUSH_BUTTON, device menu, "+sMenu+", "+sAccess, pos);
    dynInsertAt(menuList, "SEPARATOR", pos);
  }
  */
  /** e.g.: add all the device actions 
  dyn_string dsUserAccess;
  int pos,i, len, iMenu=UN_POPUPMENU_MIN;
  string sMenu;
  
  pos = dynContains(menuList, "PUSH_BUTTON, Device configuration, 10, 1");
  if(pos > 0) {
    // get the allowed device actions for the user logged in
    plcIcemonPlc_ButtonUserAccess(sDpName, "IcemonPlc", dsUserAccess);
    // remove Select if it is in the list
    len = dynContains(dsUserAccess, UN_POPUPMENU_SELECT_TEXT);
    if(len > 0)
      dynRemove(dsUserAccess, len);
    len = dynlen(dsUserAccess);
    if(len > 0)
      dynInsertAt(menuList, "SEPARATOR", pos);

    // add all the device actions
    for(i=1;i<=len;i++)
    {
      sprintf(sMenu, "%d", (iMenu+1));
      iMenu++;
      dynInsertAt(menuList, "PUSH_BUTTON, "+dsUserAccess[i]+", "+sMenu+", 1", pos);
    }
    if(len > 0)
      dynInsertAt(menuList, "SEPARATOR", pos);
  }
  */
  /** e.g.: add all the device actions depending on the current device value
  dyn_string dsUserAccess;
  int pos,i, len, iMenu=UN_POPUPMENU_MIN;
  string sMenu, sAccess, sMenuText;
  int iData;
  bool bSelected;
  bool bActionOk;
  dyn_string dsButtons = makeDynString("AckAlarm", "AckDefault", "Set"); 
  
  pos = dynContains(menuList, "PUSH_BUTTON, Device configuration, 10, 1");
  if(pos > 0) {
    // get the allowed device actions for the user logged in
    plcIcemonPlc_ButtonUserAccess(sDpName, "IcemonPlc", dsUserAccess);
    bSelected = dynContains(menuList, "PUSH_BUTTON, Deselect, 1, 1");
    dpGet(sDpName+".ProcessInput.iData", iData);
    // remove Select if it is in the list
    len = dynContains(dsUserAccess, UN_POPUPMENU_SELECT_TEXT);
    if(len > 0)
      dynRemove(dsUserAccess, len);
    len = dynlen(dsButtons);
    if(dynlen(dsUserAccess) > 0)
      dynInsertAt(menuList, "SEPARATOR", pos);

    // add the device actions
    for(i=1;i<=len;i++)
    {
      sprintf(sMenu, "%d", (iMenu+1));
      iMenu++;
      sAccess = "0";
      bActionOk = (dynContains(dsUserAccess, dsButtons[i]) > 0);  // User access
      switch(dsButtons[i])
      {
        case "..":
          bActionOk = bActionOk & bSelected & (iData == "..");
          sMenuText = "Action ..";
          break;
        default:
          bActionOk = false;
          sMenuText = "";
          break;
      }
      if(bActionOk)
        sAccess = "1";
      if(sMenuText != "")
        dynInsertAt(menuList, "PUSH_BUTTON, "+sMenuText+", "+sMenu+", "+sAccess, pos);
    }
    if(dynlen(dsUserAccess) > 0)
      dynInsertAt(menuList, "SEPARATOR", pos);
  }
  */
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc rigthClick IcemonPlc "+sDpName, "plcIcemonPlc_MenuConfiguration", sDpName, sDpType, dsAccessOk, menuList);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_HandleMenu
/** handle the answer of the popup menu
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param deviceName input, device DP name
@param sDpType input, device type
@param menuList input, the access control 
@param iMenuAnswer input, selected menu value 
*/
plcIcemonPlc_HandleMenu(string deviceName, string sDpType, dyn_string menuList, int iMenuAnswer)
{
  string sDeviceName = unGenericDpFunctions_getAlias(deviceName);
  int iUN_POPUPMENU_MIN = UN_POPUPMENU_MIN;
  dyn_string exceptionInfo;
  dyn_string dsTemp, dsSplit;
  int len, i, iMenu;

  // handle unicos menu
  if(iMenuAnswer<iUN_POPUPMENU_MIN)
    unGenericObject_HandleUnicosMenu(deviceName, sDpType, menuList, iMenuAnswer, exceptionInfo);
  else if (iMenuAnswer == 333){ //LAn DB Info
    icemon_handleMenu(sDeviceName);
  }
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc rigthClick IcemonPlc "+deviceName, "plcIcemonPlc_HandleMenu", deviceName, sDpType, menuList, iMenuAnswer);
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_SquareAnimation
/** animate the device Square widget
!!!!! function trigger by exec call, $-param and variable of the widget cannot be used, all the necessary data 
must be given to the function. The use of global var (global keyword declaration) is allowed
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param blobData input, device data
@param bSystemConnected input, connection state of the system, true=system connected/false=system not connected
@param sWidgetType input, the type of widget
@param iFESystemIntegrityAlarmValue input, front-end device system integrity alarm value
@param bFESystemIntegrityAlarmEnabled input, front-end device system integrity enable value
*/
plcIcemonPlc_SquareAnimation(string sDpN, string sData, string sType,string sDpFsm, string fsmState, bool bSystemConnected, string sWidgetType, int iFESystemIntegrityAlarmValue, bool bFESystemIntegrityAlarmEnabled)
{
  string color;
  /*!!!!!!!! all parameters must be given: function launched by exec ***/
  // encapsulate all the setValue or function call with the following if(bSystemConnected)
  if(bSystemConnected) {
    dyn_string sDpFsmArr = strsplit(sDpFsm, ".");
    string alias;
    dpGet(sDpFsmArr[1]+".tnode", alias);
    strreplace(alias, getSystemName(), "");
    fwCU_getStateColor(alias, fsmState, color);
    setMultiValue("body", "backCol", color);
  }
 
}

//---------------------------------------------------------------------------------------------------------------------------------------

// plcIcemonPlc_SquareDisconnection
/** animate the the device Square widget when the system is disconnected 
!!!!! function trigger by exec call, $-param and variable of the widget cannot be used, all the necessary data 
must be given to the function. The use of global var (global keyword declaration) is allowed

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  Ui, CTRL

@param sWidgetType input, the type of widget
*/
plcIcemonPlc_SquareDisconnection(string sWidgetType)
{
  // set all the graphical element of the widget
  setMultiValue("body", "backCol", "unDataNoAccess",
                "WarningText", "text", "",
                "WidgetArea", "visible", false,
                "SelectArea", "foreCol", "",
                "LockBmp", "visible", false);
  unGenericDpFunctions_debugTN(DEBUG_ONLY_BIT0, "plc widget IcemonPlc Square", "plcIcemonPlc_SquareDisconnection", sWidgetType);
}

//---------------------------------------------------------------------------------------------------------------------------------------

//@}
